@@ -105,7 +105,12 @@ ao_play_chunk(struct audio_output *ao, const struct music_chunk *chunk)
 
 			/* don't automatically reopen this device for
 			   10 seconds */
+			g_mutex_lock(ao->mutex);
+
+			assert(ao->fail_timer == NULL);
 			ao->fail_timer = g_timer_new();
+
+			g_mutex_unlock(ao->mutex);
 			return false;
 		}
 
@@ -192,10 +197,18 @@ static gpointer audio_output_task(gpointer arg)
 
 		case AO_COMMAND_OPEN:
 			assert(!ao->open);
-			assert(ao->fail_timer == NULL);
 			assert(ao->pipe != NULL);
 			assert(ao->chunk == NULL);
 
+			if (ao->fail_timer != NULL) {
+				/* this can only happen when this
+				   output thread fails while
+				   audio_output_open() is run in the
+				   player thread */
+				g_timer_destroy(ao->fail_timer);
+				ao->fail_timer = NULL;
+			}
+
 			error = NULL;
 			ret = ao_plugin_open(ao->plugin, ao->data,
 					     &ao->out_audio_format,
@@ -268,6 +281,16 @@ static gpointer audio_output_task(gpointer arg)
 			ao->chunk = NULL;
 			if (ao->open)
 				ao_plugin_cancel(ao->plugin, ao->data);
+
+			/* we must clear the notification now, because
+			   the notify_wait() call below must wait
+			   until audio_output_all_cancel() has cleared
+			   the pipe; if another notification happens
+			   to be still pending, we get a race
+			   condition with a crash or an assertion
+			   failure */
+			notify_clear(&ao->notify);
+
 			ao_command_finished(ao);
 
 			/* the player thread will now clear our music
